/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.apache.tuscany.sca.assembly.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.tuscany.sca.assembly.Binding;
import org.apache.tuscany.sca.assembly.Callback;
import org.apache.tuscany.sca.assembly.ComponentService;
import org.apache.tuscany.sca.assembly.Reference;
import org.apache.tuscany.sca.interfacedef.InterfaceContract;
import org.apache.tuscany.sca.policy.PolicySet;

/**
 * Represents a reference.
 * 
 * @version $Rev: 679082 $ $Date: 2008-07-23 14:13:46 +0100 (Wed, 23 Jul 2008) $
 */
public class ReferenceImpl extends AbstractReferenceImpl implements Reference, Cloneable {
    private List<Binding> bindings = new ArrayList<Binding>();
    private boolean wiredByImpl;
    private List<PolicySet> policySets = new ArrayList<PolicySet>();
    private List<ComponentService> targets = new ArrayList<ComponentService>();
    private Callback callback;
    private List<PolicySet> applicablePolicySets = new ArrayList<PolicySet>();
    private boolean promotionOverride;

    public List<PolicySet> getApplicablePolicySets() {
        return applicablePolicySets;
    }

    /**
     * Constructs a new reference.
     */
    protected ReferenceImpl() {
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        ReferenceImpl clone = (ReferenceImpl)super.clone();
        clone.bindings = new ArrayList<Binding>();
        clone.bindings.addAll(getBindings());
        clone.targets = new ArrayList<ComponentService>();
        clone.targets.addAll(getTargets());
        return clone;
    }

    public List<Binding> getBindings() {
        return bindings;
    }

    public <B> B getBinding(Class<B> bindingClass) {
        for (Binding binding : bindings) {
            if (bindingClass.isInstance(binding)) {
                return bindingClass.cast(binding);
            }
        }
        return null;
    }

    public <B> B getCallbackBinding(Class<B> bindingClass) {
        if (callback != null) {
            for (Binding binding : callback.getBindings()) {
                if (bindingClass.isInstance(binding)) {
                    return bindingClass.cast(binding);
                }
            }
        }
        return null;
    }

    public boolean isWiredByImpl() {
        return wiredByImpl;
    }

    public void setWiredByImpl(boolean wiredByImpl) {
        this.wiredByImpl = wiredByImpl;
    }

    public boolean isPromotionOverride() {
        return promotionOverride;
    }

    public void setPromotionOverride(boolean promotionOverride) {
        this.promotionOverride = promotionOverride;
    }

    public List<PolicySet> getPolicySets() {
        return policySets;
    }

    public List<ComponentService> getTargets() {
        return targets;
    }

    public Callback getCallback() {
        return callback;
    }

    public void setCallback(Callback callback) {
        this.callback = callback;
    }
    
    public void setPolicySets(List<PolicySet> policySets) {
        this.policySets = policySets; 
    }
    
    /**
     * By default return the interface contract for the reference
     */
    public InterfaceContract getInterfaceContract(Binding binding){
        return getInterfaceContract();
    }      

}
