/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.apache.tuscany.sca.contribution.impl;

import org.apache.tuscany.sca.contribution.Artifact;
import org.apache.tuscany.sca.contribution.Contribution;
import org.apache.tuscany.sca.contribution.ContributionFactory;
import org.apache.tuscany.sca.contribution.ContributionMetadata;
import org.apache.tuscany.sca.contribution.DefaultExport;
import org.apache.tuscany.sca.contribution.DefaultImport;
import org.apache.tuscany.sca.contribution.DeployedArtifact;


/**
 * Default implementation of a contribution model factory.
 * 
 * @version $Rev: 664356 $ $Date: 2008-06-07 16:35:02 +0100 (Sat, 07 Jun 2008) $
 */
public class ContributionFactoryImpl implements ContributionFactory {
    
    protected ContributionFactoryImpl() {
    }
    
    public Contribution createContribution() {
        return new ContributionImpl();
    }
    
    public ContributionMetadata createContributionMetadata() {
        return new ContributionMetadataImpl();
    }

    public Artifact createArtifact() {
        return new ArtifactImpl();
    }
    
    public DefaultExport createDefaultExport() {
        return new DefaultExportImpl();
    }
    
    public DefaultImport createDefaultImport() {
        return new DefaultImportImpl();
    }
    
    @Deprecated
    public DeployedArtifact createDeployedArtifact() {
        class DeployedArtifactImpl extends ArtifactImpl implements DeployedArtifact {
        }
        return new DeployedArtifactImpl();
    }
}
